==> arte7-0.0.2.tar.gz: arte7-0.0.2/arte7.py <==
#!/use/bin/env python
# -*- coding: utf-8 -*-

"""A simple command line tool for interacting with the arte+7 database.

arte is a great TV station and they even provide most of their programs
for 7 days after the initial airing.  But their website is truly horrible.
So this program might be useful for anyone who wants to enjoy arte
without using their website.
"""

import os
import json
import argparse
import codecs
from time import time

try:
	from urllib2 import urlopen
	import BaseHTTPServer
except ImportError:
	from urllib.request import urlopen
	import http.server as BaseHTTPServer

CACHE_DIR = os.path.expanduser('~/.cache/arte7')

if not os.path.exists(CACHE_DIR):
	os.mkdir(CACHE_DIR)


def get_unicode(url):
	"""Wrapper around urlopen() that returns a unicode string."""
	fh = urlopen(url)
	# hacky, but works ok
	encoding = fh.headers.get('content-type').split('=')[-1]
	string = fh.read()
	fh.close()
	return string.decode(encoding)


def get_cached(url):
	"""Wrapper around get_unicode() that uses a fs cache."""
	cache_key = str(hash(url))
	path = os.path.join(CACHE_DIR, cache_key)
	if not os.path.exists(path):
		s = get_unicode(url)
		with codecs.open(path, 'w', 'utf8') as fh:
			fh.write(s)
	else:
		with codecs.open(path, 'r', 'utf8') as fh:
			s = fh.read()
	return s


def clear_cache(ttl=60*60*24*7):
	"""Clear the cache used in `get_cached()`."""
	for filename in os.listdir(CACHE_DIR):
		path = os.path.join(CACHE_DIR, filename)
		mtime = os.path.getmtime(path)
		if mtime + ttl < time():
			os.unlink(path)


def get_single(vid, lang):
	"""Return description, meta and stream_url for a single video."""
	lang_map = {
		'de': 'D',
		'fr': 'F',
	}
	l = lang_map[lang]
	url = ('http://arte.tv/papi/tvguide/videos/stream/player/'
		'%s/%s_PLUS7-%s/ALL/ALL.json' % (l, vid, l))
	data = json.loads(get_cached(url))
	data = data['videoJsonPlayer']

	desc = data.get('VDE')
	meta = data.get('infoProg')
	stream_url = data['VSR']['HTTP_REACH_EQ_1']['url']

	return desc, meta, stream_url


def _filter(video, q):
	"""Return true if video mathes the search query q."""
	def match(hay, needle):
		if hay is None:
			return False
		else:
			return hay.lower().find(needle.lower()) >= 0

	return match(video.get('title'), q) or match(video.get('desc'), q)


def get_all(lang, q=''):
	url = 'http://www.arte.tv/guide/%s/plus7.json' % lang
	data = json.loads(get_unicode(url))
	videos = []
	for video in (v for v in data['videos'] if _filter(v, q)):
		v = {
			'title': video.get('title'),
			'airdate': video.get('airdate_long'),
			'image_url': video.get('image_url'),
		}
		v['desc'], v['meta'], v['stream_url'] = get_single(video['em'], lang)
		videos.append(v)
	return videos


def format_console(videos):
	"""Format `videos` for displaying in the console."""
	ret = []
	for video in videos:
		ret.append(
			"%(title)s - %(airdate)s - %(meta)s\n"
			"%(desc)s\n%(stream_url)s" % video)
	return '\n\n'.join(ret)


def format_html(videos):
	"""Format `videos` as  HTML."""
	css = """
		.videos {
			list-style: none;
			margin: 0;
			padding: 0;
		}
		.video {
			border: 1px solid silver;
			padding: 1em;
			margin: 2em 0;
			overflow: hidden;  /* clearfix */
		}
		.video img {
			float: left;
			width: 30%;
			margin-right: 1em;
		}
		/* colors */
		a:link {
			color: #f9471b;
		}
		html {
			background-color: #202020;
			color: white;
			font-family: ProximaNova, Arial;
			padding: 1em;
		}
		body {
			background-color: transparent;
			max-width: 70em;
			margin: 0 auto;
		}
		.video {
			background-color: #ecebe9;
			color: #282828;
		}
	"""
	html_tpl = """<!DOCTYPE html>
		<html>
			<head>
				<meta charset="utf8" />
				<title>ARTE+7</title>
				<style>%(css)s</style>
			</head>
			<body>
				<h1>Arte+7</h1>
				<ul class="videos">%(list)s</ul>
			</body>
		</html>"""
	item_tpl = """<li class="video">
			<h2><a href="%(stream_url)s">%(title)s</a></h2>
			<img src="%(image_url)s" alt="" />
			<p>%(desc)s</p>
			<span class="airdata">%(airdate)s</span>
			-
			<span class="meta">%(meta)s</span>
		</li>"""
	return html_tpl % {
		'list': '\n'.join((item_tpl % v for v in videos)),
		'css': css,
	}


class HTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
	"""HTTP Request Handler showing a single HTML page."""

	html = ''

	def do_GET(self):
		self.send_response(200)
		self.send_header("Content-type", 'text/html')
		self.end_headers()
		self.wfile.write(self.html.encode('utf8'))


def start_server(port):
	"""Start HTTP Server with HTTPRequestHandler on port `port`."""
	server_address = ('localhost', port)
	httpd = BaseHTTPServer.HTTPServer(server_address, HTTPRequestHandler)
	sa = httpd.socket.getsockname()
	print("Serving HTTP on", sa[0], "port", sa[1], "...")
	httpd.serve_forever()


def parse_args():
	"""Parse command linee arguments."""
	parser = argparse.ArgumentParser(description=__doc__)
	parser.add_argument('-l', '--language', choices=['de', 'fr'], default='de')
	parser.add_argument('-t', '--to',
		choices=['console', 'html', 'serve', 'json'],
		default='console', help="output format")
	parser.add_argument('-p', '--port', type=int, help="port used for serve",
		default=8000)
	parser.add_argument('q', nargs='?', help="search term", default='')
	return parser.parse_args()


def main():
	"""Entry point for the command line tool."""
	args = parse_args()
	videos = get_all(args.language, q=args.q)

	if args.to == 'html':
		print(format_html(videos))
	elif args.to == 'serve':
		HTTPRequestHandler.html = format_html(videos)
		start_server(args.port)
	elif args.to == 'json':
		print(json.dumps(videos, indent=2))
	else:
		print(format_console(videos))

	clear_cache()


if __name__ == '__main__':
	main()

==> arte7-0.0.3.tar.gz: arte7-0.0.3/arte7.py <==
#!/use/bin/env python
# -*- coding: utf-8 -*-

"""A simple command line tool for interacting with the arte+7 database.

arte is a great TV station and they even provide most of their programs
for 7 days after the initial airing.  But their website is truly horrible.
So this program might be useful for anyone who wants to enjoy arte
without using their website.
"""

import os
import json
import argparse
import codecs
from time import time

try:
	from urllib2 import urlopen
	import BaseHTTPServer
except ImportError:
	from urllib.request import urlopen
	import http.server as BaseHTTPServer

CACHE_DIR = os.path.expanduser('~/.cache/arte7')

if not os.path.exists(CACHE_DIR):
	os.mkdir(CACHE_DIR)


def get_unicode(url):
	"""Wrapper around urlopen() that returns a unicode string."""
	fh = urlopen(url)
	# hacky, but works ok
	encoding = fh.headers.get('content-type').split('=')[-1]
	string = fh.read()
	fh.close()
	return string.decode(encoding)


def get_cached(url):
	"""Wrapper around get_unicode() that uses a fs cache."""
	cache_key = str(hash(url))
	path = os.path.join(CACHE_DIR, cache_key)
	if not os.path.exists(path):
		s = get_unicode(url)
		with codecs.open(path, 'w', 'utf8') as fh:
			fh.write(s)
	else:
		with codecs.open(path, 'r', 'utf8') as fh:
			s = fh.read()
	return s


def clear_cache(ttl=60*60*24*7):
	"""Clear the cache used in `get_cached()`."""
	for filename in os.listdir(CACHE_DIR):
		path = os.path.join(CACHE_DIR, filename)
		mtime = os.path.getmtime(path)
		if mtime + ttl < time():
			os.unlink(path)


def get_single(vid, lang):
	"""Return description, meta and stream_url for a single video."""
	lang_map = {
		'de': 'D',
		'fr': 'F',
	}
	l = lang_map[lang]
	url = ('http://arte.tv/papi/tvguide/videos/stream/player/'
		'%s/%s_PLUS7-%s/ALL/ALL.json' % (l, vid, l))
	data = json.loads(get_cached(url))
	data = data['videoJsonPlayer']

	desc = data.get('VDE')
	meta = data.get('infoProg')
	try:
		stream_url = data['VSR']['HTTP_REACH_EQ_1']['url']
	except KeyError:
		stream_url = data['VSR']['HTTP_MP4_EQ_1']['url']

	return desc, meta, stream_url


def _filter(video, q):
	"""Return true if video mathes the search query q."""
	def match(hay, needle):
		if hay is None:
			return False
		else:
			return hay.lower().find(needle.lower()) >= 0

	return match(video.get('title'), q) or match(video.get('desc'), q)


def get_all(lang, q=''):
	url = 'http://www.arte.tv/guide/%s/plus7.json' % lang
	data = json.loads(get_unicode(url))
	videos = []
	for video in (v for v in data['videos'] if _filter(v, q)):
		v = {
			'title': video.get('title'),
			'airdate': video.get('airdate_long'),
			'image_url': video.get('image_url'),
		}
		v['desc'], v['meta'], v['stream_url'] = get_single(video['em'], lang)
		videos.append(v)
	return videos


def format_console(videos):
	"""Format `videos` for displaying in the console."""
	ret = []
	for video in videos:
		ret.append(
			"%(title)s - %(airdate)s - %(meta)s\n"
			"%(desc)s\n%(stream_url)s" % video)
	return '\n\n'.join(ret)


def format_html(videos):
	"""Format `videos` as  HTML."""
	css = """
		.videos {
			list-style: none;
			margin: 0;
			padding: 0;
		}
		.video {
			border: 1px solid silver;
			padding: 1em;
			margin: 2em 0;
			overflow: hidden;  /* clearfix */
		}
		.video img {
			float: left;
			width: 30%;
			margin-right: 1em;
		}
		/* colors */
		a:link {
			color: #f9471b;
		}
		html {
			background-color: #202020;
			color: white;
			font-family: ProximaNova, Arial;
			padding: 1em;
		}
		body {
			background-color: transparent;
			max-width: 70em;
			margin: 0 auto;
		}
		.video {
			background-color: #ecebe9;
			color: #282828;
		}
	"""
	html_tpl = """<!DOCTYPE html>
		<html>
			<head>
				<meta charset="utf8" />
				<title>ARTE+7</title>
				<style>%(css)s</style>
			</head>
			<body>
				<h1>Arte+7</h1>
				<ul class="videos">%(list)s</ul>
			</body>
		</html>"""
	item_tpl = """<li class="video">
			<h2><a href="%(stream_url)s">%(title)s</a></h2>
			<img src="%(image_url)s" alt="" />
			<p>%(desc)s</p>
			<span class="airdata">%(airdate)s</span>
			-
			<span class="meta">%(meta)s</span>
		</li>"""
	return html_tpl % {
		'list': '\n'.join((item_tpl % v for v in videos)),
		'css': css,
	}


class HTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
	"""HTTP Request Handler showing a single HTML page."""

	html = ''

	def do_GET(self):
		self.send_response(200)
		self.send_header("Content-type", 'text/html')
		self.end_headers()
		self.wfile.write(self.html.encode('utf8'))


def start_server(port):
	"""Start HTTP Server with HTTPRequestHandler on port `port`."""
	server_address = ('localhost', port)
	httpd = BaseHTTPServer.HTTPServer(server_address, HTTPRequestHandler)
	sa = httpd.socket.getsockname()
	print("Serving HTTP on", sa[0], "port", sa[1], "...")
	httpd.serve_forever()


def parse_args():
	"""Parse command line arguments."""
	parser = argparse.ArgumentParser(description=__doc__)
	parser.add_argument('-l', '--language', choices=['de', 'fr'], default='de')
	parser.add_argument('-t', '--to',
		choices=['console', 'html', 'serve', 'json'],
		default='console', help="output format")
	parser.add_argument('-p', '--port', type=int, help="port used for serve",
		default=8000)
	parser.add_argument('q', nargs='?', help="search term", default='')
	return parser.parse_args()


def main():
	"""Entry point for the command line tool."""
	args = parse_args()
	videos = get_all(args.language, q=args.q)

	if args.to == 'html':
		print(format_html(videos))
	elif args.to == 'serve':
		HTTPRequestHandler.html = format_html(videos)
		start_server(args.port)
	elif args.to == 'json':
		print(json.dumps(videos, indent=2))
	else:
		print(format_console(videos))

	clear_cache()


if __name__ == '__main__':
	main()
